"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const commandParams = require("../../command-params");
const helpers_1 = require("../../helpers");
const proxy_base_command_1 = require("./proxy-base-command");
const proxySetCommandName = "proxy|set";
class ProxySetCommand extends proxy_base_command_1.ProxyCommandBase {
    constructor($errors, $injector, $prompter, $analyticsService, $logger, $proxyService) {
        super($analyticsService, $logger, $proxyService, proxySetCommandName);
        this.$errors = $errors;
        this.$injector = $injector;
        this.$prompter = $prompter;
        this.$analyticsService = $analyticsService;
        this.$logger = $logger;
        this.$proxyService = $proxyService;
        this.allowedParameters = [
            new commandParams.StringCommandParameter(this.$injector),
            new commandParams.StringCommandParameter(this.$injector),
            new commandParams.StringCommandParameter(this.$injector),
            new commandParams.StringCommandParameter(this.$injector)
        ];
    }
    execute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let hostname = args[0], port = args[1], username = args[2], password = args[3];
            const noHostName = !hostname;
            const noPort = !port || !this.isValidPort(port);
            if (!helpers_1.isInteractive() && (noHostName || noPort || this.isPasswordRequired(username, password))) {
                this.$errors.fail("Console is not interactive - you need to supply all command parameters.");
            }
            if (noHostName) {
                hostname = yield this.$prompter.getString("Hostname", { allowEmpty: false });
            }
            if (noPort) {
                if (port) {
                    this.$logger.warn(this.getInvalidPortMessage(port));
                }
                port = yield this.getPortFromUserInput();
            }
            if (!username) {
                this.$logger.info("In case your proxy requires authentication, please specify username and password. If authentication is not required, just leave it empty.");
                username = yield this.$prompter.getString("Username", { defaultAction: () => "" });
            }
            if (this.isPasswordRequired(username, password)) {
                password = yield this.$prompter.getPassword("Password");
            }
            if (username && password) {
                yield this.$proxyService.setCredentials({
                    password,
                    username
                });
            }
            const proxyCache = {
                PROXY_HOSTNAME: hostname,
                PROXY_PORT: +port
            };
            this.$proxyService.setCache(proxyCache);
            this.$logger.out("Successfully setup proxy.");
            yield this.tryTrackUsage();
        });
    }
    isPasswordRequired(username, password) {
        return !!(username && !password);
    }
    isValidPort(port) {
        const parsedPortNumber = parseInt(port);
        return !isNaN(parsedPortNumber) && parsedPortNumber > 0 && parsedPortNumber < 65536;
    }
    getPortFromUserInput() {
        return __awaiter(this, void 0, void 0, function* () {
            const schemaName = "port";
            const schema = {
                message: "Port",
                type: "input",
                name: schemaName,
                validate: (value) => {
                    return (!value || !this.isValidPort(value)) ? this.getInvalidPortMessage(value) : true;
                }
            };
            const prompterResult = yield this.$prompter.get([schema]);
            return prompterResult[schemaName];
        });
    }
    getInvalidPortMessage(port) {
        return `Specified port ${port} is not valid. Please enter a value between 1 and 65535.`;
    }
}
exports.ProxySetCommand = ProxySetCommand;
$injector.registerCommand(proxySetCommandName, ProxySetCommand);
